@@ -11,12 +11,26 @@ class socket_talker(object):
     def __init__(self, host, port):
         self.host = host
         self.port = port
-        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        self.sock.settimeout(3)
-        try:
-       	    self.sock.connect((host, port))
-       	except socket.error, msg:
-       	    print 'Cannot connect to MPD:', msg
+        # Try to connect via IPv6 if supported, but on failure
+        # fall back to IPv4. This helps not only in cases with
+        # broken IPv6 networking (fairly common), but also works 
+        # in cases where someone explicitly uses an IPv4 address
+        # for the host, without having to explicitly check for it.
+        self.sock = None
+        if socket.has_ipv6:
+            try:
+                self.sock = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
+                self.sock.settimeout(3)
+                self.sock.connect((host, port))
+            except socket.error, msg:
+                self.sock = None
+        if not self.sock:
+            self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            self.sock.settimeout(3)
+            try:
+                self.sock.connect((host, port))
+            except socket.error, msg:
+                print 'Cannot connect to MPD:', msg
         self.file = self.sock.makefile("rb+")
         self.current_line = ''
         self.ack = ''
@@ -122,15 +136,17 @@ commands = {
     ("plchanges",     1): ('%s %d', MANY, '', plitem_delim), # <playlist version>
     ("pause",         1): ('%s %d', ZERO, '', []), # <bool pause>
 
-    ("update",      1): ('%s "%s"', ONE, 'update', []), # <string path>
-    ("listall",     1): ('%s "%s"', MANY, '', plitem_delim), # <string path>
-    ("listallinfo", 1): ('%s "%s"', MANY, '', plitem_delim), # <string path>
-    ("lsinfo",      1): ('%s "%s"', MANY, '', plitem_delim), # <string directory>
-    ("add",         1): ('%s "%s"', ZERO, '', []), # <string>
-    ("load",        1): ('%s "%s"', ZERO, '', []), # <string name>
-    ("rm",          1): ('%s "%s"', ZERO, '', []), # <string name>
-    ("save",        1): ('%s "%s"', ZERO, '', []), # <string playlist name>
-    ("password",    1): ('%s "%s"', ZERO, '', []), # <string password>
+    ("update",        1): ('%s "%s"', ONE, 'update', []), # <string path>
+    ("listall",       1): ('%s "%s"', MANY, '', plitem_delim), # <string path>
+    ("listallinfo",   1): ('%s "%s"', MANY, '', plitem_delim), # <string path>
+    ("lsinfo",        1): ('%s "%s"', MANY, '', plitem_delim), # <string directory>
+    ("add",           1): ('%s "%s"', ZERO, '', []), # <string>
+    ("load",          1): ('%s "%s"', ZERO, '', []), # <string name>
+    ("rm",            1): ('%s "%s"', ZERO, '', []), # <string name>
+    ("save",          1): ('%s "%s"', ZERO, '', []), # <string playlist name>
+    ("password",      1): ('%s "%s"', ZERO, '', []), # <string password>
+    ("playlistclear", 1): ('%s "%s"', ZERO, '', []), # <string playlist name>
+    
 
     ("move",   2): ("%s %d %d", ZERO, '', []), # <int from> <int to>
     ("moveid", 2): ("%s %d %d", ZERO, '', []), # <int songid from> <int to>
@@ -139,16 +155,17 @@ commands = {
     ("seek",   2): ("%s %d %d", ZERO, '', []), # <int song> <int time>
     ("seekid", 2): ("%s %d %d", ZERO, '', []), # <int songid> <int time>
 
-    # <string type> <string what>
-    ("find",   2): ('%s "%s" "%s"', MANY, '', plitem_delim),
-    ("rename", 2): ('%s "%s" "%s"', ZERO, '', []),
+    ("find",           2): ('%s "%s" "%s"', MANY, '', plitem_delim),
+    ("rename",         2): ('%s "%s" "%s"', ZERO, '', []),
+    ("playlistadd",    2): ('%s "%s" "%s"', ZERO, '', []), # <str playlist name> <str path>
+    ("playlistdelete", 2): ('%s "%s" "%s"', ZERO, '', []), # <str playlist name> <int song id>
+    
+    ("playlistmove", 3): ('%s "%s" "%s" "%s"', ZERO, '', []), # <str playlist name> <ing song id> <int position>
 
     # <string type> <string what>
     ("search", 2): ('%s "%s" "%s"', MANY, '', plitem_delim),
     ("search", 4): ('%s "%s" "%s" "%s" "%s"', MANY, '', plitem_delim),
 
-    # list <metadata arg1> [<metadata arg2> <search term>]
-
     # <metadata arg1>
     ("list", 1): ('%s "%s"', MANY, '', litem_delim),
 
